#!BPY
""" Registration info for Blender menus:
Name: 'BZW Toolkit v2.0.1'
Blender: 240
Group: 'Object'
Tooltip: 'Add and edit BZflag style objects within blender.'
"""
__author__ = ("'Teppic'")
__version__ = "0.1"
__bpydoc__ = """\
This script helps exports to BZW 2.04 format.
Usage:
Run this script from "Object/Bzwtoolkit" menu.
"""
# 
# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import Draw, BGL, Object
from bzflag import BZtypeList
from bzflag import BZsceneWriter,BZconfig

class BZWTOOLKIT:

	def __init__(self):
		self.objectlist={} 	 		#list of all objects withou bzflag variables
		self.BZobjectlist={} 		#list of objects with bzflag variables set
		self.BZmatobjectlist={}		#list of 'bzflag' defined materials
		self.BZmatIndexList={}		#backwardly readble above instance, yeah I could retrive it but can't be arsed
		self.telelist = {}   		#List of tele objects referencable by index number
		self.menulist=''		    #As above but for Draw.Menu function calls
		self.BZmenulist=''			#As above but for Draw.Menu function calls
		self.BAmatmenuList=''		#As above but for Draw.Menu function calls
		self.indexnumber=1			#for correlation between selectedobject.val
		self.BZindexnumber=1		#for correlation between selectedBZobject.val
		self.BZmatindexnumber=1		#for correlation between selectedBZmatobject.val
		self.selectedObject='1'	 	#set the default index, will become Draw.Menu class object
		self.selectedBZObject='1'	#set the default index, will become Draw.Menu class object
		self.selectedBZmatObject='1'#set the default index, will become Draw.Menu class object
		self.typeCache={}			#holds number versus type for dynamic gui objects
		self.message="Hello...."	#can't remember.......
		self.materialval={}			#holds default values, then menu values
		self.material={}			#holds menu objects
		self.stringval={}			#holds default values, then string values
		self.string={}				#holds string objects
		self.attribval={}			#holds default values, then string values
		self.attrib={}				#holds string objects
		self.bool={}				#holds toggle objects FIXME, should be done via event switching
		self.boolval={}				#holds default values, then bool states
		self.AddObjectList=(	'world','waterLevel','dynamicColor','textureMatrix','material','physics',
							'define','group','meshbox','meshpyr','arc','cone','sphere','box','pyramid',
							'link','teleporter','base','weapon','zone')
		self.ExceptList=('size','position','rotation','name')					

#########
#main gui
#########
	def event(self, evt, val):
		if evt==Draw.ESCKEY and val==1:
			Draw.Exit()
			return
		elif val != 0:        
			return
		else: 
			return
			
	def button_event(self,evt):
		if evt == 1:
			Draw.Exit()
			return
		elif evt == 2:
			self.addNew()
			return
		elif evt == 4:
			self.editExisting()
			return
		elif evt==5:
			obj=Blender.Object.Get(self.objectlist[int(self.selectedObject.val)])
			self.template=BZtypeList.mesh()
			self.ExceptList=('spin','shear','scale','shift','name','position','size','rotation')
			for t in self.template.bools:
				if t not in self.ExceptList:self.propertyCheck(obj,t,getattr(self.template,t,False))
			for t in self.template.strings:
				if t not in self.ExceptList:self.propertyCheck(obj,t,getattr(self.template,t,''))
			for t in self.template.attribs:
				if t not in self.ExceptList:self.propertyCheck(obj,t,getattr(self.template,t,''))
			self.propertyCheck(obj,'bzflag','mesh')
			self.propertyCheck(obj,'name',obj.name)
			self.getobjects(Blender.Scene.getCurrent())	
		elif evt==99:
			self.getobjects(Blender.Scene.getCurrent())	
		return

	def gui(self):
		BGL.glClearColor(0.5,0.5,0.5,1)	
		BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
		BGL.glColor3f(1,1,1)
		Draw.Button("Exit", 1, 10, 10, 65, 25,"Exit the toolkit")
		Draw.Button("Add", 2, 10, 75, 120, 20,"Add a new BZflag object to the scene")
		Draw.Button("Edit" , 4, 140, 75, 120, 20,"Edit an existing BZflag objects properties")
		self.selectedBZObject=Draw.Menu(self.BZmenulist, 3, 140, 45, 120, 20, self.BZindexnumber ,"Select the Object to change")
		self.selectedObject=Draw.Menu(self.menulist,5,270,45,120,20,self.indexnumber,'Select the mesh to "BZFlagify"')
		Draw.Button("Mesh Export", 5, 270, 75, 120, 20,"Set the export flag on an existing mesh")
		BGL.glRasterPos2i(10, 50)
		Draw.Text(self.message)
		Draw.Button("Update", 99, 325, 10, 65, 25,"Update the object lists")
######################################################
#add gui
######################################################
	def addgui(self):
		BGL.glClearColor(0.5,0.5,0.5,1)	
		BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
		BGL.glColor3f(1,1,1)
		BGL.glRasterPos2i(5, 190)
		Draw.Text("Objects:")
		BGL.glRasterPos2i(120, 210)		
		Draw.Text("Select the item you would like to add:")			
		for a in range(0,len(self.AddObjectList)):
			Draw.Button(self.AddObjectList[a],a+1,5+((a-((a/5)*5))*110), 160-(30*((a/5))),100,20)
		Draw.Button("Back", 100, 10, 10, 65, 25,"Exit to the toolkit")

	def addevent(self,evt,val):
		if val != 0:
			if evt==Draw.ESCKEY:
				self.getobjects(Blender.Scene.getCurrent())	
	def addbutton_event(self,evt):
		if evt==0:
			pass
		elif evt==100:
			self.getobjects(Blender.Scene.getCurrent())
		else:
			obj=BZtypeList.Objects[self.AddObjectList[evt-1]]()
			obj.name=self.checkName(obj.name)
			try:
				mesh=Blender.Mesh.Get(obj.name)
				mesh.name='defucnt'
				print mesh.name
			except:
				pass
			module=BZsceneWriter.Object[obj.type()]
			writer=module.importer(obj)
			writer.run()		
			
######################################################
#edit gui
######################################################
	def editgui(self):
		BGL.glClearColor(0.5,0.5,0.5,1)	
		BGL.glClear(BGL.GL_COLOR_BUFFER_BIT)
		BGL.glColor3f(1,1,1)
		BGL.glRasterPos2i(5, 190)
		Draw.Text("Editing options for: %s" % self.template.name)		
		a=0
		for tog in self.template.bools:
			if tog not in self.ExceptList:
				Draw.Toggle(tog,a+1,5+((a-((a/5)*5))*110), 160-(30*((a/5))),100,20,self.boolval[tog])
				self.typeCache[a+1]=tog
				a+=1
		for st in self.template.strings:
			if st not in self.ExceptList:
				if st=='matref':
					if not self.stringval[st]:
						self.stringval[st]=1
					elif type(self.stringval[st])=='str':#convert to a number to reference against the menu list
						self.stringval[st]=[self.BZmatIndexList[self.stringval[st]]]
					self.string[st]=Draw.Menu('matref%s' % self.BZmatmenulist,a+1,5+((a-((a/5)*5))*110), 160-(30*((a/5))),100,20, self.stringval[st] ,"Select the objects material")
					self.typeCache[a+1]=st
					a+=1
				else:
					self.string[st]=Draw.String(st+': ',a+1,5+((a-((a/5)*5))*110), 160-(30*((a/5))),100,20, self.stringval[st],100)
					self.typeCache[a+1]=st
					a+=1
		for mat in self.template.materials:
			self.material[mat]=Draw.Menu('%s%s'	% (mat,self.BZmatmenulist), a+1, 5+((a-((a/5)*5))*110),
											160-(30*((a/5))),100,20, self.materialval[mat] ,"Select faces' material")
			self.typeCache[a+1]=mat
			a+=1
		for st in self.template.attribs:
			if st not in self.ExceptList:
				self.attrib[st]=Draw.String(st+': ',a+1,5+((a-((a/5)*5))*110), 160-(30*((a/5))),100,20, self.attribval[st],100)
				self.typeCache[a+1]=st
				a+=1									
		Draw.Button("Cancel", 100, 10, 10, 65, 25,"Exit to the toolkit")
		Draw.Button("Apply", 99, 490, 10, 65, 25,"Apply the changes")

	def editevent(self,evt,val):
		if val != 0:
			if evt==Draw.ESCKEY:
				self.getobjects(Blender.Scene.getCurrent())	
	def editbutton_event(self,evt):
		try:
			data=self.typeCache[evt]
		except KeyError:
			data=None
		if evt==0:
			return
		elif evt==100:
			self.getobjects(Blender.Scene.getCurrent())
		elif evt==99:
			self.applyExisting()
		elif data:
			if data in self.template.materials:
				indval=self.material[data].val
				self.materialval[data]=int(indval)
			elif data =='matref':
				indval=self.string[data].val
				self.stringval[data]=int(indval)
			elif data in self.template.strings:
				self.stringval[data]=self.string[data].val
			elif data in self.template.attribs:
				self.attribval[data]=self.attrib[data].val
			elif data in self.template.bools:
				self.boolval[data]=1-self.boolval[data]
		Draw.Redraw(1)
		return

######################################################
#self.funtions()
######################################################

	def addNew(self):
		Draw.Register(self.addgui, self.addevent, self.addbutton_event)
		Draw.Redraw()
	
	def editExisting(self):
		self.boolval={}#reset values
		self.bool={}
		self.stringval={}
		self.string={}
		self.materialval={}
		self.material={}
		self.attribval={}
		self.attrib={}#reset values
		obj=Blender.Object.Get(self.BZobjectlist[int(self.selectedBZObject.val)])
		BZType=self.propertyCheck(obj,'bzflag')
		if BZType:	self.template=BZtypeList.Objects[BZType]()
		else:		return
		if BZType in ['meshpyr','meshbox','arc','cone','sphere']:
			self.ExceptList=('spin','shear','scale','shift','name','position','size','rotation','ingroup')
		elif BZType in ['material']:
			self.ExceptList=('addtexture','texture','matref','name','ingroup')
		elif BZType =='mesh':
			self.ExceptList=('spin','shear','scale','shift','name','ingroup','position','size','rotation')
		else:
			self.ExceptList=('size','position','rotation','name','tilt','height','ingroup')
		for t in self.template.bools:
			if t not in self.ExceptList:	self.boolval[t]=self.propertyCheck(obj,t)
		for t in self.template.strings:
			if t not in self.ExceptList:	self.stringval[t]=self.propertyCheck(obj,t)
		for t in self.template.attribs:
			if t not in self.ExceptList:	self.attribval[t]=self.propertyCheck(obj,t)
		for t in self.template.materials:
			data=self.propertyCheck(obj,t)
			if data:				self.materialval[t]=self.BZmatIndexList[data]
			else:					self.materialval[t]=1
		self.template.name=obj.name
		Draw.Register(self.editgui, self.editevent, self.editbutton_event)
		Draw.Redraw()

	def applyExisting(self):
		obj=Blender.Object.Get(self.BZobjectlist[int(self.selectedBZObject.val)])
		self.template=BZtypeList.Objects[self.propertyCheck(obj,'bzflag')]()
		for t in self.template.bools:
			if t not in self.ExceptList:self.propertyCheck(obj,t,self.boolval[t])
		for t in self.template.strings:
			if t not in self.ExceptList:self.propertyCheck(obj,t,self.stringval[t])
		for t in self.template.attribs:
			if t not in self.ExceptList:self.propertyCheck(obj,t,self.attribval[t])
		for t in self.template.materials:
			data=self.BZmatobjectlist[self.materialval[t]]
			self.propertyCheck(obj,t,data)
		#reset base colours
		if self.template.type()=='base':
			colour=int(self.cleanStr(self.propertyCheck(obj,'color')))
			if colour in [1,2,3,4]:
				wallTextures=['','red_basewall.png','green_basewall.png','blue_basewall.png','purple_basewall.png']
				topTextures=['','red_basetop.png','green_basetop.png','blue_basetop.png','purple_basetop.png']
				colours=['','red','green','blue','purple']
				try:
					wallimage=Blender.Image.Get(wallTextures[colour])
				except NameError:
					wallimage=Blender.Image.Load(BZconfig.bztexpath+wallTextures[colour])
				try:
					topimage=Blender.Image.Get(topTextures[colour])
				except NameError:
					topimage=Blender.Image.Load(BZconfig.bztexpath+topTextures[colour])
				mesh=obj.getData()
				for i in xrange(2,6):
					mesh.faces[i].image=wallimage
				mesh.faces[1].image=topimage
				mesh.update()
		self.getobjects(Blender.Scene.getCurrent())	
			

	def propertyCheck(self, object, name ,value="none"):	
		try:
			attrib=object.getProperty(name).getData()
			if value =="none":
				return attrib
			else:
				object.getProperty(name).setData(value)
				return
		except:
			if value =="none":
				return ''
			else:
				object.addProperty(name,value)
				return


	def getobjects(self, scene):
		try:
			material=Blender.Material.Get('BZtexface')
		except NameError:
			material=Blender.Material.New('BZtexface')
			material.setMode('TexFace')
			material.setSpec(0.1)
		objects = []
		objects = scene.getChildren()
		self.usedNames=[]
		self.menulist="Object: %t"
		self.BZmenulist="Object: %t"
		self.BZmatmenulist=": %t|None %x 1"
		self.BZmatobjectlist[1]='None'
		self.BZmatIndexList['None']=1
		self.baseMatMenu="%t|None %x 1"
		i=j=1
		k=2
		for obj in objects:
			self.usedNames.append(obj.name)
			if self.propertyCheck(obj,'bzflag'):
				self.BZmenulist=self.BZmenulist +"|%s " %(obj.name) + " %x" + str(j)
				self.BZobjectlist[j]=obj.name
				j+=1
				if self.propertyCheck(obj,'bzflag')=='material':
					self.BZmatmenulist=self.BZmatmenulist +"|%s " %(obj.name) + " %x" + str(k)
					self.BZmatobjectlist[k]=obj.name
					self.BZmatIndexList[obj.name]=k
					k+=1					
			elif obj.getType()=='Mesh' and not obj.getParent() and 1 in obj.layers:
				self.menulist=self.menulist +"|%s " %(obj.name) + " %x" + str(i)
				self.objectlist[i]=obj.name
				i+=1
		mat=Blender.Material.Get()
		Draw.Register(self.gui, self.event, self.button_event) #thunderbirds are go!!!!
		
			
	def checkName(self,name):
		newname=name
		a=1
		while newname in self.usedNames:
			newname='%s_%s' %(name,a)
			a+=1
		self.usedNames.append(newname)
		return newname
	
	
	
	
	def bitCheck(self,dec,bit):
		bitVars={}
		for i in range(0,20):	
			b=19-i
			if (dec-(2**b)) >= 0:
				bitVars[b]=1
				dec -=(2**b)
			else:
				bitVars[b]=0
		return bitVars[(bit)]
	def cleanStr(self,string):
		string=str(string)
		string=string.replace('\'','')
		string=string.replace('[','')
		string=string.replace(']','')
		string=string.replace(',','')
		return string
############# Main Program #############	

if Blender.Get('version') < 240:
    print "Warning: BZW Tool Box can't run in this Blender version!"
    print " You aren't running blender version >= 2.40"
    print " Upgrade your version from http://blender.org/"
else:
    #print "Startup"
    scene = Blender.Scene.getCurrent()
    run=BZWTOOLKIT()
    run.getobjects(scene)
