"""bzcommon module """
# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
import BZtypeList
import Blender
import math
from Blender import Mathutils
from math import cos,sin,sqrt 

def propertyCheck(obj, name ,value=None):
		try:
			attrib=obj.getProperty(name).getData()
			if value ==None:
				##print "Propertycheck returned " + str(attrib)+" for "+str(name)+" in "+str(obj.name)
				return attrib
			else:
				obj.getProperty(name).setData(value)
				##print "Property " +obj.name+"."+ name +" set to " + str(value)
				return
		except:
			if value ==None:
				##print "Propertycheck found no property with the name "+ obj.name +"."+ name
				return None
			else:
				##print "Added property " +obj.name+"."+ name +" set to " + str(value)
				obj.addProperty(name,value)
				return

class setProperties:
	
	def __init__(self,bzobj):
		obj=Blender.Object.Get(bzobj.name)
		propertyCheck(obj,'bzflag',bzobj.type())
		if bzobj.type() in ['meshpyr','arc','meshbox','sphere','tetra','cone']:
			ExceptList=['']
		else:
			ExceptList=['size','position','rotation']
		for string in bzobj.strings:
			propertyCheck(obj, string,getattr(bzobj, string, None))
		for attrib in bzobj.attribs:
			if attrib in ExceptList:
				pass
			else:
				propertyCheck(obj, attrib, ''.join(str(getattr(bzobj, attrib, None))))
		for bool in bzobj.bools:
			propertyCheck(obj, bool, getattr(bzobj, bool, False))

def deg2rad(deg):
		degs=float(deg)
		pi=6.2831853071795862
		rads=(degs*pi)/360.0
		return rads

def rad2deg(rad):
		rads=float(rad)
		pi=6.2831853071795862
		degs=(rads*360.0)/pi
		return degs
		
def spin2matrix(tfm):		
		a=float(tfm[0])
		x=float(tfm[1])
		y=float(tfm[2])
		z=float(tfm[3])
		v=Mathutils.Vector([x,y,z])
		m=Mathutils.RotationMatrix(a,3,'r',v)
		return m
		
def retList(line):
	retlist=[]
	if not line: return retlist
	line=line.replace('\t',' ')
	line=line.replace(',',' ')
	while line.find('  ')>=0:
		line=line.replace('  ',' ')
	line=line.strip()
	line=line + ' '
	while line!='':
		retlist.append(line[:line.find(' ')])
		line=line[line.find(' ')+1:]
	return retlist	
	
def rnd(inp,frac):
	input=float(inp)
	out=round(input,frac)
	dec=str(out-int(out))
	num=str(int(out))
	dec=dec[2:frac+2]
	stout='%s.%s' %(num,dec)
	stout=stout.replace('..','.')
	if input<0 and float(stout)>0:
		stout='-%s' % stout
	return stout


	
	
	
