"""bzflag.reader module, read bzw file, split into objetcs, parse object parameters
	to object class instance, import object, create blender scene"""
# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
import BZtypeList
import BZsceneWriter
import Blender
import sys
class reader:
	def __init__(self):
		self.debug=1
		self.groupList={}
		self.inGroup=False
		self.importedNames=[]
		self.index=0
		self.lineNo=0
		self.matlist=[]
		self.telelist=[]
		self.telelist.append(0)
		try:
			material=Blender.Material.Get('BZtexface')
		except NameError:
			material=Blender.Material.New('BZtexface')
			material.setMode('TexFace')
			material.setSpec(0.1)
		self.matlist.append('BZtexface')
	#enddef
	def read(self,filename):
		inobject=False
		inface=False
		meshtext=0
		self.pipe=open(filename,'r')
		for line in self.pipe.readlines():
			self.lineNo+=1
			keywd=str(retList(line)[0])
			if keywd:
				key1=keywd[0].lower()
				keywd=key1+keywd[1:]
			datalst=retList(line)[1:]
			datastr=' '.join(datalst)
			if self.debug>1:print '%s:%s' % (self.lineNo,line)
			if keywd=='matref' and not meshtext:						#numbered materials
				try:
					Blender.Material.Get(datastr)
				except NameError:
					try:
						q=int(datastr)
						matname=str(self.matlist[q])
						Blender.Material.Get(matname)
						datastr=matname
					except:
						print 'Unknown Material %s' % datastr
						keywd='#'
			########################################################################
			######################## Import start proper ###########################
			
			if keywd=='':		pass	#skip whitespace
			elif keywd[0]=='#': pass	#It's a comment, skip it
			
			# ##################################################
			elif self.inGroup and keywd=='enddef':
				self.currentGroup.objects=self.groupList[self.currentGroup.name]
				if self.debug >2: print 'group objects are %s' % self.groupList[self.currentGroup.name]
				module=BZsceneWriter.define
				writer=module.importer(self.currentGroup)
				writer.run()
				if self.debug:print '\033[1;32mFinished importing group:-\033[0m %s' % self.currentGroup.name
				self.inGroup=False
				setattr(BZtypeList.define,'objects',[])
				#self.groupList[self.currentGroup.name]=[]
			
			# ##################################################
			elif not inobject:#check keyword to start an object#
				try:
					obj=BZtypeList.Objects[keywd]()
					inobject=True
					self.index+=1
					if self.debug:print '\033[1;34mStarting import object of type:-\033[0m '+obj.type()
					if obj.type()=='define':
						inobject=self.startGroup(retList(line))
						obj.name=datastr
					if obj.type()=='options':
						self.startoptions()
					if obj.type()=='teleporter':obj.telenum=self.teleStart(retList(line))
					if obj.type()=='group': 
						obj.parent=datastr
						obj.name=['%s_i%s' % (datastr,self.index)]
				except:
					if self.debug:print 'Unknown object type %s :Skipping' % keywd
					pass
				#endtry
			
			# ###############################################
			elif inface:
				if keywd in face.strings:
					setattr(face, keywd,datastr)
				elif keywd in face.attribs:
					setattr(face, keywd,datalst)
				elif keywd.lower() in face.bools:
					setattr(face, keywd.lower(),True)
				elif keywd=='endface':
					obj.face.append(face)
					inface=False
					if self.debug>1:print '\033[1;32mFinished importing face:-\033[0m'
				else:
					if self.debug:print '\033[1;33mUnknown attribute \033[0m%s\033[1;33m for Object:-\033[0m face' % (keywd)				
			
			# ################################################
			elif inobject:
				if keywd=='from':keywd='ffrom'## cheap python reserved word workaround
				if keywd=='diffuse':keywd='color'## cheap colour alias workaround, some poeple need to learn to spell!!
				if keywd=='type':keywd='ttype'##w-weapon type alias
				if obj.type()=='dynamicColor' and keywd!='name' and keywd!='end':		#change keyword for dyncol 
					keywd1=str(retList(line)[0])
					keywd2=str(retList(line)[1])
					keywd='_'.join([keywd1,keywd2])
					datalst=retList(line)[2:]
					datastr=' '.join(datalst)
				if obj.type()=='options':
					self.opt.write(line)
				if meshtext:                  		#store drawType options for imported meshes
					self.mtex.write(line)
					if keywd in ['lod','matref']:
						meshtext+=1
					elif keywd=='end':
						meshtext-=1					#end
				# ##########
				elif keywd=='face':					#start instance of class face
					face=BZtypeList.face()
					inface=True						#end
				# ##########
				elif keywd=='drawInfo':				#start to store drawType options for imported meshes
					meshtext=1
					if obj.name==BZtypeList.Objects[obj.type()]().name:	#check has proper name
						obj.name=['%s_%s' % (obj.name,self.index)]
					try:
						self.importedNames.index(obj.name)				#check has unique name
						obj.name=['%s_%s' % (obj.name,self.index)]
					except:
						pass
					self.mtex=Blender.Text.New(obj.name)
					self.mtex.write('drawInfo\n')	#ready to store text
				# ###########      proper import section   #############
				elif keywd in obj.strings:
					setattr(obj,keywd,datastr)
				
				elif keywd in obj.attribs:
					setattr(obj,keywd,datalst)
				
				elif keywd in obj.params:
					lis=getattr(obj,keywd,[])
					app=tuple(datalst)
					lis.append(app)
					setattr(obj,keywd,lis)
				
				elif keywd.lower() in obj.bools:
					setattr(obj,keywd.lower(),True)
				
				elif keywd in obj.materials:
					datastr=datastr[datastr.find(' ')+1:]     						#remove matref
					try:
						Blender.Material.Get(datastr)
						setattr(obj,keywd,datastr)
					except NameError:
						try:
							q=int(datastr)
							matname=str(self.matlist[q])
							Blender.Material.Get(matname)
							datastr=matname
							setattr(obj,keywd,datastr)
						except:
							print 'Unknown Material %s' % datastr
				
				elif keywd=='end':
					inobject=False
					if len(obj.name)>15:											#make name short enough to be used in blender
						obj.name=obj.name[:15]
					if obj.name==BZtypeList.Objects[obj.type()]().name:				#check not std
						obj.name='%s_%s' % (obj.name,self.index)
					try:
						self.importedNames.index(obj.name)							#check unique
						obj.name='%s_%s' % (obj.name,self.index)
					except:
						self.importedNames.append(obj.name)							#add to list
					if self.inGroup:
						self.groupList[self.currentGroup.name].append(obj.name)		#append to glist
					try:
						module=BZsceneWriter.Object[obj.type()]
						writer=module.importer(obj)
						writer.run()
					except KeyError:
						pass ##special object types must not be registered.
					if obj.type()=='material':
						self.matlist.append(obj.name)								#register material
					for param in obj.params:
						setattr(BZtypeList.Objects[obj.type()],param,[])			#reset class instance, Fuck knows why?? it's an instance ffs
					if self.debug:print '\033[1;32mFinished importing object:-\033[0m %s' % obj.name
				else:
					if self.debug and obj.type()!='options':print '\033[1;33mUnknown attribute \033[0m%s\033[1;33m for Object:-\033[0m %s' % (keywd,obj.type())
				#endif
			else:
				pass
			#endif
		#endfor
		self.pipe.close()
		Blender.Window.EditMode(0)
		Blender.Window.EditMode(1)
		Blender.Window.EditMode(0)
		Blender.Scene.GetCurrent().update()
	#enddef
	def startoptions(self):
		self.opt=Blender.Text.New('prefix')
		self.opt.write('options\n')
		return
	def teleStart(self,start):
		try:
			name=start[1]
		except NameError:
			print '\033[1;31mError: On map line %s, teleporter with no name!\033[0m' % self.lineno
			fatalError()
		try:
			self.importedNames.index(name)
			print '\033[1;31mError: On map line %s, two teleporters with same name!\033[0m' % self.lineno
			fataError()
		except:
			pass
		self.telelist.append(name)
		telenum=self.telelist.index(name)
		return telenum
	#enddef
	def startGroup(self,start):
		if self.inGroup:
			print '\033[1;31mError: On map line %s, nested group definition!\033[0m' % self.lineno
			fatalError()
		try:
			name=' '.join(start[1:])
		except IndexError:
			print '\033[1;31mError: On map line %s, group definition with no name!\033[0m' % self.lineno
			fatalError()
		if self.groupList.has_key(name):
			print '\033[1;31mError: On map line %s, group definition allready exists!\033[0m' % self.lineno
			fatalError()
		self.groupList[name]=[]
		self.inGroup=True
		self.currentGroup=BZtypeList.define()
		self.currentGroup.name=name
		return False
	#enddef			
#endclass
def retList(line):
	retlist=[]
	line=line.replace('\t',' ')
	#line=line.replace(',',' ')  not needed??
	while line.find('  ')>=0:
		line=line.replace('  ',' ')
	line=line.strip()
	line=line + ' '
	while line!='':
		retlist.append(line[:line.find(' ')])
		line=line[line.find(' ')+1:]
	return retlist
#enddef
def fatalError():
	print '\033[1;31m#####################################################'
	print '#######  Exited on fatal map file error!  ###########'
	print '#####################################################\033[0m'
	sys.exit()
#enddef
