"""module box, takes a bzobject from the parser and creates a blender object."""

# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import Material, Texture, NMesh, Image, Object
import BZconfig
import BZtypeList
import BZcommon
import BZobject
from BZobject import bzprim,BlObject


class importer(bzprim):

	def __init__(self,bzobj):
		try:
			self.wallimage=Image.Get('boxwall.png')
		except NameError:
			self.wallimage=Blender.Image.Load(BZconfig.bztexpath+'boxwall.png')
		try:
			self.topimage=Blender.Image.Get('roof.png')
		except NameError:
			self.topimage=Blender.Image.Load(BZconfig.bztexpath+'roof.png')
		self.obj=bzobj
	
	def run(self):
		obj=NMesh.New(self.obj.name)
		obj.addMaterial(Blender.Material.Get('BZtexface'))
		obj.hasFaceUV(1)
		for verts in self.obj.verts:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		for face in self.obj.faces:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			f.flag=1
			f.mode=5
			f.mat=0
			f.image=self.wallimage
			f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]
			obj.faces.append(f)
		obj.faces[1].image=self.topimage
		obj.faces[0].image=self.topimage
		scobj=Object.New('Mesh',self.obj.name)
		scobj.link(obj)
		scene = Blender.Scene.getCurrent ()
		scene.link (scobj)
		self.updateto()	
		Blender.Redraw()

class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.box()
		self.obj=blobj
	
	def run(self):
		ro=BZconfig.ro
		self.updateFrom()
		self.pipe.write('box\n')
		self.pipe.write('\tname %s\n' % self.out.name)
		self.writeBools()
		self.writeStrings()
		self.writeAttribs()
		self.writePrim()
		self.pipe.write('end\n\n')
		self.pipe.flush()
