"""module meshpyr, takes a bzobject from the parser and creates a blender object."""

# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import NMesh, Image, Object
import BZconfig
import BZcommon
import math
from math import sin, cos 
from BZobject import bzmesh,BlObject
import BZtypeList

class importer(bzmesh):

	def __init__(self,bzobj):
		try:
			self.wallimage=Image.Get('boxwall.png')
		except NameError:
			self.wallimage=Blender.Image.Load(BZconfig.bztexpath+'boxwall.png')
		self.obj=bzobj
		self.matlib={}
	
	def run(self):
		obj=NMesh.New(self.obj.name)
		obj.addMaterial(Blender.Material.Get('BZtexface'))
		i=1
		if self.obj.matref:
			obj.addMaterial(Blender.Material.Get(self.obj.matref))
			i=2
		for mat in self.obj.materials:
			cmat=getattr(self.obj,mat,None)
			if cmat:
				try:
					obj.addMaterial(Blender.Material.Get(cmat))	
					self.matlib[mat]=i
					i+=1
				except AttributeError:
					j=0
					for psmat in obj.getMaterials(-1):
						if psmat.name==cmat:self.matlib[mat]=j
						j+=1
		if not self.obj.flatshading:
			obj.mode=32
		obj.hasFaceUV(1)
		a=float(BZcommon.deg2rad(self.obj.angle[0]))
		d=int(self.obj.divisions[0])
		da=float(a/d)
		mx,my,mz=self.obj.size
		mx=float(mx)
		my=float(my)
		mz=float(mz)
		dx,dy,dz=self.obj.position
		dx=float(dx)
		dy=float(dy)
		dz=float(dz)
		vertlist=[(dx,dy,dz),(dx,dy,dz+1*mz)]
		facelist=[]
		for sect in range(0,d+1):
			ca=float(sect*da)+BZcommon.deg2rad(float(self.obj.rotation[0]))
			ix=cos(ca)*mx+dx
			iy=sin(ca)*my+dy
			vertlist.append((ix,iy,0+dz))
		for verts in vertlist:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)			
		face=(0,2,1)
		f=NMesh.Face()		
		for verts in face:	f.v.append(obj.verts[verts])
		if self.matlib.has_key('startside'):#startface
			f.mat=self.matlib['startside']
		elif	self.obj.matref:
			f.mat=1
		else:
			f.mat=0
			f.image=self.wallimage
			f.flag=1
		f.mode=5
		f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
		obj.faces.append(f)		
		facelist=[]
		for sect in range (0,d):#side faces
			o=sect
			facelist.append((1,o+2,o+3))
		for face in facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			if self.matlib.has_key('edge'):
				f.mat=self.matlib['edge']
			elif	self.obj.matref:
				f.mat=1
			else:
				f.mat=0
				f.image=self.wallimage
				f.flag=1
				f.mode=5
			f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
			obj.faces.append(f)	
		facelist=[]
		face=((1,o+3,0))
		f=NMesh.Face()
		for verts in face:	f.v.append(obj.verts[verts])
		if self.matlib.has_key('endside'):#endface
			f.mat=self.matlib['endside']
		elif	self.obj.matref:
			f.mat=1
		else:
			f.mat=0
			f.image=self.wallimage
			f.flag=1
			f.mode=5
		f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
		obj.faces.append(f)		
		facelist=[]

		for sect in range (0,d):#bottom
			o=sect
			facelist.append((o+3,o+2,0))#bottomface
		for face in facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			if self.matlib.has_key('bottom'):
				f.mat=self.matlib['bottom']
			elif	self.obj.matref:
				f.mat=1
			else:
				f.mat=0
				f.image=self.wallimage
				f.flag=1
			f.mode=5
			f.uv=[(0.0,1.0),(0.0,0.0),(1.0,0.0),(1.0,1.0)]		
			obj.faces.append(f)	
		scobj=Object.New('Mesh',self.obj.name)
		scobj.link(obj)
		scene = Blender.Scene.getCurrent ()
		scene.link (scobj)
		self.updateto()	
		Blender.Redraw()

class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.cone()
		self.obj=blobj
	
	def run(self):
		ro=BZconfig.ro
		self.updateFrom()
		self.pipe.write('cone\n')
		self.pipe.write('\tname %s\n' % self.out.name)
		self.writeMaterials()
		self.writeBools()
		self.writeStrings()
		self.writeAttribs()
		self.writeMesh()
		self.pipe.write('end\n\n')
		self.pipe.flush()
		
		
