# material module
# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import Material, Texture, Object,NMesh
import BZcommon
import BZconfig
from BZcommon import propertyCheck,rnd
from matTemp import sphere
from BZobject import BlObject
import BZcolourList
import BZtypeList
import os


class importer:
	def __init__(self,bzobj):
		#added for windows cache file path fix
		self.delim='/'
		try: 
			a=os.getenv('os')
			print a
			if 'Win' in a:
				self.delim='\\'
		except AttributeError:
			pass			
		#end added for windows cache file path fix##
		coldict=BZcolourList.Colours
		self.obj=bzobj
		if self.obj.matref:
			self.copymaterial()
		else:
			self.mat=Blender.Material.New(self.obj.name)
		if self.obj.addtexture and not self.obj.texture:##update soon for multitextures.
			self.obj.texture=self.obj.addtexture
		## strip http://blah.blah.blah
		if self.obj.texture:
			line=self.obj.texture
			while line.find('/')!=-1:
				line=line[line.find('/')+1:]
				##print line
			self.texFileName=line
			try:
				image=Blender.Image.Get(self.texFileName)
			except NameError:
				if (self.obj.texture[:7]=='http://') | (self.obj.texture[:6]=='ftp://'):
					if (self.obj.texture[:7]=='http://'):
						try:
							texFileName='http%s%s' % (self.delim,self.obj.texture[7:])
							#added for windows cache file path fix
							if self.delim=='\\':
								texFileName=texFileName.replace('/',self.delim)
							#end added for windows cache file path fix	
							image=Blender.Image.Load('%s%s' % (BZconfig.bzcachepath,texFileName))
							#print 'ok cache http'
						except IOError:
							try:
								image=Blender.Image.Load('%s%s' % (BZconfig.cstexpath,self.texFileName))
								#print 'ok custom from http'
							except IOError:
								print'\033[1;33mUnable to load texture %s\033[0m ' % (self.texFileName)
								image=None
					else:
						try:
							texFileName='ftp%s%s' % (self.delim,self.obj.texture[6:])
							#added for windows cache file path fix
							if self.delim=='\\':
								texFileName=texFileName.replace('/',self.delim)
							#end added for windows cache file path fix	
							image=Blender.Image.Load('%s%s' % (BZconfig.bzcachepath,texFileName))
							#print 'ok cache ftp'
						except IOError:
							try:
								image=Blender.Image.Load('%s%s' % (BZconfig.cstexpath,self.texFileName))
								#print 'ok custom from ftp'
							except IOError:
								print'\033[1;33mUnable to load texture %s\033[0m ' % (self.texFileName)
								image=None
				else:
					try:
						image=Blender.Image.Load('%s%s' % (BZconfig.cstexpath,self.texFileName))
						print 'ok custom'
					except IOError:
						try:
							image=Blender.Image.Load('%s%s' % (BZconfig.bztexpath,self.texFileName))
						except IOError:
							try:
								image=Blender.Image.Load('%s%s.png' % (BZconfig.bztexpath,self.texFileName))
							except IOError:
								print'\033[1;33mUnable to load texture %s\033[0m ' % (self.texFileName)
								image=None
			try:
				self.texture=Blender.Texture.Get(self.texFileName)
			except NameError:
				self.texture=Blender.Texture.New(self.texFileName)
			self.texture.setType('Image')
			if image:self.texture.image=image
		else:
			self.texture=False
		if self.obj.color:
			if self.obj.color[0] in coldict:
				setattr(self.obj,'color',coldict[self.obj.color[0]])
			try:
				float(self.obj.color[0])
			except:
				self.obj.color=[1,1,1,1]
	#enddef

	
	def run(self):
		obj=Blender.NMesh.New('%s' % self.obj.name)
		if self.obj.ambient:
			try:
				a=float(self.obj.ambient[3])
			except IndexError:
				a=0
			self.mat.setAmb(a)
			print '\033[1;33mBlenders ambient settings are taken from the Blender world environment\n\
Your custom ambient colour settings will not be visible.\n\
They will, however, be stored until export.\033[0m'
		else:
			self.mat.setAmb(0)
		if self.obj.specular:
			r=float(self.obj.specular[0])
			g=float(self.obj.specular[1])
			b=float(self.obj.specular[2])
			r=0.5*r
			g=0.5*g
			b=0.5*b
			self.mat.setSpecCol((r,g,b))
			self.mat.setSpec(2.0)
			try:
				self.mat.setSpecTransp(1-float(self.obj.specular[3]))
			except IndexError:
				pass
		else:
			self.mat.setSpecCol((0,0,0))
		if self.obj.color:
			r=float(self.obj.color[0])
			g=float(self.obj.color[1])
			b=float(self.obj.color[2])
			self.mat.setRGBCol((r,g,b))
			try:
				self.mat.setAlpha(float(self.obj.color[3]))
			except IndexError:
				pass
		else:
			self.mat.setRGBCol((0,0,0))
		if self.obj.shininess:
			h=int(float(self.obj.shininess[0])*2)
			if not h:h=1
			self.mat.setHardness(h)
		if self.obj.emission:
			print '\033[1;33mBlenders emit settings are taken from the objects material\n\
Your custom emission colour settings will not be visible.\n\
They will, however, be stored until export.\033[0m'
		if self.texture and not self.obj.notextures:
			self.mat.setTexture(0,self.texture)
		obj.addMaterial(self.mat)
		for verts in sphere.vertlist:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		for face in sphere.facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
				f.mat=0
				f.mode=32
			obj.faces.append(f)
		#obj.mode=32
		scobj=Blender.Object.New('Mesh','%s' % self.obj.name)
		scobj.setDrawType(3)
		#scobj.setDrawMode(128)
		scobj.link(obj)
		scene = Blender.Scene.getCurrent ()
		scene.link (scobj)
		propertyCheck(scobj,'bzflag',self.obj.type())
		exceptList=('color','specular','texture','addtexture','shininess')
		for attrib in self.obj.attribs:
			if attrib not in exceptList:
				propertyCheck(scobj, attrib, getattr(self.obj, attrib, None))
		for bool in self.obj.bools:
			propertyCheck(scobj, bool, getattr(self.obj, bool, False))
		for string in self.obj.strings:
			if string not in exceptList:
				propertyCheck(scobj, string, getattr(self.obj, string, False))
		scobj.layers=[20]
		Blender.Redraw()	
		
		
	def copymaterial(self):
		pass

class export(BlObject):
	def __init__(self,blobj,pipe):
		self.out=BZtypeList.material()
		self.obj=blobj
		self.pipe=pipe
		self.mat=blobj.getData().getMaterials()[0]
		
	def run(self):
		ro=BZconfig.ro
		self.updateFrom()
		self.out.name=self.mat.name
		self.pipe.write('material\n\tname %s\n' % self.out.name)
		self.pipe.flush()
		#colour#
		r,g,b=self.mat.rgbCol
		a=self.mat.getAlpha()
		if r or g or b:
			self.pipe.write('\tdiffuse %s %s %s %s\n' %(rnd(r,ro),rnd(g,ro),rnd(b,ro),rnd(a,ro)))
			self.pipe.flush()
		r,g,b=self.mat.specCol
		s=self.mat.spec/2
		r=r*s;g=g*s;b=b*s		
		if r or g or b:
			a=1-self.mat.specTransp
			self.pipe.write('\tspecular %s %s %s %s\n' %(rnd(r,ro),rnd(g,ro),rnd(b,ro),rnd(a,ro)))
		if self.mat.hard:
			s=self.mat.hard/2
			self.pipe.write('\tshininess %s\n' % rnd(s,ro))
		if self.mat.getTextures()[0]:#export texture
			mtex=self.mat.getTextures()[0]
			if mtex.tex.getType()!='Image':
				print '\033[1;31mError: Textures of type "%s" cannot be exported!\033[0m' % mtex.tex.getType()
			else:
				if not mtex.tex.image.name:
					print '\033[1;31mError: No image assigned to material %s!\033[0m' % self.out.name	
				elif mtex.tex.image.name in BZconfig.KnownTextures:
					filename=mtex.tex.image.name
				else:
					filename='%s%s' % (BZconfig.cstexhost,mtex.tex.image.name)
				self.pipe.write('\ttexture %s\n' % filename)
		self.writeStrings()
		self.writeAttribs()
		self.writeBools()
		self.pipe.write('end\n\n')
		self.pipe.flush()
		
	def updateFrom(self):
		for prop in self.obj.getAllProperties():
			if prop.name in self.out.attribs:
				if propertyCheck(self.obj,prop.name):
					data=self.cleanStr(propertyCheck(self.obj,prop.name))
					setattr(self.out,prop.name,data)
			elif prop.name in self.out.bools:
				if propertyCheck(self.obj,prop.name):
					setattr(self.out,prop.name,True)
			elif prop.name in self.out.strings:
				if propertyCheck(self.obj,prop.name):
					data=self.cleanStr(propertyCheck(self.obj,prop.name))
					setattr(self.out,prop.name,data)
		
		
