"""sphere.py bzflag i/o module for sphere objects"""

# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****


import Blender
from Blender import NMesh, Image, Object
import BZconfig
import BZcommon
from math import pi, cos, sin
from BZobject import bzmesh,BlObject
import BZtypeList

class importer(bzmesh):

	def __init__(self,bzobj):
		try:
			self.wallimage=Image.Get('boxwall.png')
		except NameError:
			self.wallimage=Blender.Image.Load(BZconfig.bztexpath+'boxwall.png')
		self.obj=bzobj

	def run(self):
		obj=NMesh.New(self.obj.name)
		if self.obj.matref:
			mat=Blender.Material.Get(self.obj.matref)
			obj.setMaterials([Blender.Material.Get(self.obj.matref)])
			tex=mat.getTextures()[0].tex
			image=tex.getImage()
			self.wallimage=image
			obj.hasFaceUV(1)
		else:
			obj.addMaterial(Blender.Material.Get('BZtexface'))
		if not self.obj.flatshading:
			obj.mode=32
		if self.obj.radius:
			s=float(self.obj.radius[0])
			self.obj.size=[s,s,s]# Yeah, they lose the size, but it's doing nothing!
		mx,my,mz=self.obj.size
		mx=float(mx)
		my=float(my)
		mz=float(mz)
		dx,dy,dz=self.obj.position
		dx=float(dx)
		dy=float(dy)
		dz=float(dz)
		vertlist=[(dx,dy,1*mz+dz),(dx,dy,dz-1*mz)]
		facelist=[]
		div=int(self.obj.divisions[0])
		q=0.5*pi
		for d in range(1,div+1):
			for s in range(0,4*d):
				ph=0.5*pi*s/d
				th=0.5*pi*d/div
				z=cos(th)*mz+dz
				y=sin(th)*sin(ph)*my+dy
				x=sin(th)*cos(ph)*mx+dx
				v=(x,y,z)
				vertlist.append(v)
				if d != div:
					v=(x,y,-z)
					vertlist.append(v)
		facelist=[]
		k=div-1
		ro=1+(k*k+k)*2
		fc=0
		for q in range(0,4):
		 for i in range(0,div):
		  for j in range(0,i+1):
			lstrip=q==3 and j==i
			llayer=i==div-1
			b=1+j+(i*i+i)*2+q*(i+1)
			c=b+1
			k=i-1
			if i:
				a=1+(k*k+k)*2
				if not lstrip:a+=q*(k+1)+j
			else:a=0
			if lstrip: c=1+(i*i+i)*2
			k+=2
			d=2+(k*k+k)*2+q*(k+1)+j
			a=a*2
			if llayer:b+=ro;c+=ro
			else:b=b*2;c=c*2
			if i==div-2:d+=ro
			else:d=d*2
			facelist.append((a,b,c))
			if not llayer:facelist.append((d,c,b));b+=1;c+=1
			a+=1
			if i != div-2:d+=1
			facelist.append((a,c,b))
			if not llayer:facelist.append((b,c,d))
		for verts in vertlist:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		for face in facelist:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			if self.wallimage:
				f.image=self.wallimage
				f.uv=[(0.5,1.0),(0.0,0.0),(1.0,0.0)]#,(1.0,1.0)]# Its a representation, who said it had to look nice?
				f.flag=1
				f.mode=5
			f.mat=0
			obj.faces.append(f)
		scobj=Object.New('Mesh',self.obj.name)
		scobj.link(obj)
		scene = Blender.Scene.getCurrent ()
		scene.link (scobj)
		self.updateto()
		Blender.Redraw()

class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.sphere()
		self.obj=blobj
	
	def run(self):
		ro=BZconfig.ro
		self.updateFrom()
		self.pipe.write('sphere\n')
		self.pipe.write('\tname %s\n' % self.out.name)
		self.writeMaterials()
		self.writeBools()
		self.writeStrings()
		self.writeAttribs()
		self.writeMesh()
		self.pipe.write('end\n\n')
		self.pipe.flush()
		
		
