"""module teleporter, takes a bzobject from the parser and creates a blender object."""

# Copyright (C) 2006: A.D.Foster
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# 
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****

import Blender
from Blender import Material, Texture, NMesh, Image, Object
import BZconfig
from BZobject import bzprim,BlObject
import BZcommon
from BZcommon import retList
import BZtypeList

class importer(bzprim):

	def __init__(self,bzobj):
		try:
			material=Material.Get('BZteleporter')
		except NameError:
			material=Blender.Material.New('BZteleporter')
			material.rgbCol = [ 0.6, 0.6, 0.6 ]
			material.setAlpha(0.2)
			material.mode = Material.Modes.ZTRANSP
		try:
			self.bimage=Image.Get('caution.png')
		except NameError:
			self.bimage=Image.Load(BZconfig.bztexpath+'caution.png')
		self.meshmat=Blender.Material.Get('BZtexface')
		self.sidemat=material
		self.obj=bzobj
	
	def run(self):
		obj=NMesh.New(self.obj.name)
		obj.addMaterial(self.sidemat)
		for verts in self.obj.verts:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		i=0
		for face in self.obj.faces:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			f.mode=4
			f.mat=0
			obj.faces.append(f)
		scobj=Blender.Object.New('Mesh',self.obj.name)
		scobj.setDrawType(3)
		scobj.setDrawMode(128)
		scobj.link(obj)
		scobj.setSize(float(self.obj.size[0]),float(self.obj.size[1]),float(self.obj.size[2])) #do before updateto() or scales border.
		scene = Blender.Scene.getCurrent ()
		scene.link (scobj)
		scobj.makeParent([self.drawBorder()])
		self.updateto()	
		Blender.Redraw()

	def drawBorder(self):
		obj=NMesh.New('%s_bdr' % self.obj.name)
		obj.addMaterial(self.meshmat)
		obj.hasFaceUV(1)	
		b=float(self.obj.border[0])/2
		x,y,z=self.obj.size
		x=float(x);y=float(y);z=float(z)
		bverts=(
					[b,-b-y,0.0],
					[b,b-y,0.0],
					[-b,b-y,0.0],
					[-b,-b-y,0.0],
					[b,-b-y,z],
					[b,b-y,z],
					[-b,b-y,z],
					[-b,-b-y,z],
					[b,-b-y,(2*b)+z],
					[-b,-b-y,(2*b)+z],
					[-b,b+y,(2*b)+z],
					[b,b+y,(2*b)+z],
					[b,b+y,0.0],
					[b,-b+y,0.0],
					[-b,-b+y,0.0],
					[-b,b+y,0.0],
					[b,b+y,z],
					[b,-b+y,z],
					[-b,-b+y,z],
					[-b,b+y,z]
				)
		bfaces=	(
					[9,3,0,8],
					[4,0,1,5],
					[5,1,2,6],
					[6,2,3,7],
					[9,8,11,10],
					[8,4,16,11],
					[5,6,18,17],
					[7,9,10,19],
					[18,14,13,17],
					[17,13,12,16],
					[11,12,15,10],
					[19,15,14,18]
				)
					
					
		for verts in bverts:
			v=NMesh.Vert(verts[0],verts[1],verts[2])
			obj.verts.append(v)
		i=0
		for face in bfaces:
			f=NMesh.Face()
			for verts in face:
				f.v.append(obj.verts[verts])
			f.flag=1
			f.mode=5
			f.mat=0
			f.image=self.bimage
			if i in (4,5,6,7):
				f.uv=[(0.0,b),(0.0,0.0),(y*2,0.0),(y*2,b)]
			else:
				f.uv=[(z,0.0),(0.0,0.0),(0.0,b),(z,b)]
			obj.faces.append(f)
			i+=1
		scobj=Blender.Object.New('Mesh','%s_bdr' % self.obj.name)
		scobj.link(obj)
		scene = Blender.Scene.getCurrent ()
		scene.link (scobj)
		return scobj
	#enddef
	
	


class export(BlObject):
	def __init__(self,blobj,pipe):
		self.pipe=pipe
		self.out=BZtypeList.teleporter()
		self.obj=blobj
	
	def run(self):
		ro=BZconfig.ro
		self.updateFrom()
		self.pipe.write('teleporter %s\n' % self.out.name)
		self.pipe.write('\tname %s\n' % self.out.name)
		self.writeBools()
		self.writeStrings()
		self.writeAttribs()
		self.writePrim()
		self.pipe.write('end\n\n')
		self.pipe.flush()
		
		
